@@ -27,6 +27,13 @@ GObject      *gdu_application_new_widget (GduApplication  *application,
                                           const gchar     *name,
                                           GtkBuilder     **out_builder);
 
+GduLocalJob  *gdu_application_create_local_job  (GduApplication *application,
+                                                 UDisksObject   *object);
+void          gdu_application_destroy_local_job (GduApplication *application,
+                                                 GduLocalJob    *job);
+GList        *gdu_application_get_local_jobs_for_object (GduApplication *application,
+                                                         UDisksObject   *object);
+
 
 G_END_DECLS
 
