@@ -27,20 +27,29 @@ enum
   GDU_DEVICE_TREE_MODEL_COLUMN_ICON,
   GDU_DEVICE_TREE_MODEL_COLUMN_NAME,
   GDU_DEVICE_TREE_MODEL_COLUMN_OBJECT,
+  GDU_DEVICE_TREE_MODEL_COLUMN_BLOCK,
   GDU_DEVICE_TREE_MODEL_COLUMN_WARNING,
   GDU_DEVICE_TREE_MODEL_COLUMN_PULSE,
   GDU_DEVICE_TREE_MODEL_COLUMN_JOBS_RUNNING,
   GDU_DEVICE_TREE_MODEL_COLUMN_POWER_STATE_FLAGS,
+  GDU_DEVICE_TREE_MODEL_COLUMN_SIZE,
+  GDU_DEVICE_TREE_MODEL_COLUMN_SELECTED,
   GDU_DEVICE_TREE_MODEL_N_COLUMNS
 };
 
 GType               gdu_device_tree_model_get_type            (void) G_GNUC_CONST;
-GduDeviceTreeModel *gdu_device_tree_model_new                 (UDisksClient       *client);
-UDisksClient       *gdu_device_tree_model_get_client          (GduDeviceTreeModel *model);
+GduDeviceTreeModel *gdu_device_tree_model_new                 (GduApplication     *application);
+GduApplication     *gdu_device_tree_model_get_application     (GduDeviceTreeModel *model);
 gboolean            gdu_device_tree_model_get_iter_for_object (GduDeviceTreeModel *model,
                                                                UDisksObject       *object,
                                                                GtkTreeIter        *iter);
 
+void                gdu_device_tree_model_clear_selected      (GduDeviceTreeModel *model);
+void                gdu_device_tree_model_toggle_selected     (GduDeviceTreeModel *model,
+                                                               GtkTreeIter        *iter);
+GList              *gdu_device_tree_model_get_selected        (GduDeviceTreeModel *model);
+GList              *gdu_device_tree_model_get_selected_blocks (GduDeviceTreeModel *model);
+
 
 G_END_DECLS
 
