@@ -11,8 +11,6 @@ G_BEGIN_DECLS
 /* enumerations from "gduenums.h" */
 GType gdu_volume_grid_element_type_get_type (void) G_GNUC_CONST;
 #define GDU_TYPE_VOLUME_GRID_ELEMENT_TYPE (gdu_volume_grid_element_type_get_type ())
-GType gdu_format_duration_flags_get_type (void) G_GNUC_CONST;
-#define GDU_TYPE_FORMAT_DURATION_FLAGS (gdu_format_duration_flags_get_type ())
 GType gdu_power_state_flags_get_type (void) G_GNUC_CONST;
 #define GDU_TYPE_POWER_STATE_FLAGS (gdu_power_state_flags_get_type ())
 G_END_DECLS
