@@ -11,7 +11,6 @@
 #define __GDU_TYPES_H__
 
 #include <gtk/gtk.h>
-#define UDISKS_API_IS_SUBJECT_TO_CHANGE
 #include <udisks/udisks.h>
 
 #include <libgdu/libgdu.h>
@@ -41,6 +40,12 @@ typedef struct _GduPasswordStrengthWidget GduPasswordStrengthWidget;
 struct _GduEstimator;
 typedef struct _GduEstimator GduEstimator;
 
+struct GduDVDSupport;
+typedef struct GduDVDSupport GduDVDSupport;
+
+struct GduLocalJob;
+typedef struct GduLocalJob GduLocalJob;
+
 G_END_DECLS
 
 #endif /* __GDU_TYPES_H__ */
