@@ -1,23 +1,10 @@
 /* -*- mode: C; c-file-style: "gnu"; indent-tabs-mode: nil; -*-
  *
- * Copyright (C) 2008-2011 Red Hat, Inc.
+ * Copyright (C) 2008-2012 Red Hat, Inc.
  *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
+ * Licensed under GPL version 2 or later.
  *
- * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
- *
- * You should have received a copy of the GNU Lesser General
- * Public License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
- * Boston, MA 02111-1307, USA.
- *
- * Author: David Zeuthen <davidz@redhat.com>
+ * Author: David Zeuthen <zeuthen@gmail.com>
  */
 
 #include "config.h"
@@ -29,7 +16,6 @@
 #include "gduwindow.h"
 #include "gduunlockdialog.h"
 #include "gduvolumegrid.h"
-#include "gduutils.h"
 
 /* From GVfs's monitor/udisks2/gvfsudisks2volume.c */
 static const SecretSchema luks_passphrase_schema =
@@ -96,9 +82,9 @@ unlock_cb (UDisksEncrypted *encrypted,
                                             res,
                                             &error))
     {
-      gdu_window_show_error (data->window,
-                             _("Error unlocking encrypted device"),
-                             error);
+      gdu_utils_show_error (GTK_WINDOW (data->window),
+                            _("Error unlocking encrypted device"),
+                            error);
       g_error_free (error);
     }
   dialog_data_free (data);
