@@ -16,6 +16,7 @@
 #include <stdlib.h>
 
 #include "gduvolumegrid.h"
+#include "gduapplication.h"
 
 /* ---------------------------------------------------------------------------------------------------- */
 
@@ -83,18 +84,18 @@ struct _GduVolumeGrid
 {
   GtkWidget parent;
 
+  GduApplication *application;
   UDisksClient *client;
   UDisksObject *block_object;
 
-  gboolean container_visible;
-  gchar *container_markup;
-
   GList *elements;
 
   GridElement *selected;
   GridElement *focused;
 
   gboolean animating_spinner;
+
+  gchar *no_media_string;
 };
 
 struct _GduVolumeGridClass
@@ -108,8 +109,9 @@ struct _GduVolumeGridClass
 enum
 {
   PROP_0,
-  PROP_CLIENT,
-  PROP_BLOCK_OBJECT
+  PROP_APPLICATION,
+  PROP_BLOCK_OBJECT,
+  PROP_NO_MEDIA_STRING,
 };
 
 enum
@@ -147,8 +149,6 @@ gdu_volume_grid_finalize (GObject *object)
 {
   GduVolumeGrid *grid = GDU_VOLUME_GRID (object);
 
-  g_free (grid->container_markup);
-
   g_signal_handlers_disconnect_by_func (grid->client,
                                         G_CALLBACK (on_client_changed),
                                         grid);
@@ -158,7 +158,9 @@ gdu_volume_grid_finalize (GObject *object)
 
   if (grid->block_object != NULL)
     g_object_unref (grid->block_object);
-  g_object_unref (grid->client);
+  g_object_unref (grid->application);
+
+  g_free (grid->no_media_string);
 
   G_OBJECT_CLASS (gdu_volume_grid_parent_class)->finalize (object);
 }
@@ -173,14 +175,18 @@ gdu_volume_grid_get_property (GObject    *object,
 
   switch (property_id)
     {
-    case PROP_CLIENT:
-      g_value_set_object (value, grid->client);
+    case PROP_APPLICATION:
+      g_value_set_object (value, grid->application);
       break;
 
     case PROP_BLOCK_OBJECT:
       g_value_set_object (value, grid->block_object);
       break;
 
+    case PROP_NO_MEDIA_STRING:
+      g_value_set_string (value, grid->no_media_string);
+      break;
+
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
       break;
@@ -197,14 +203,19 @@ gdu_volume_grid_set_property (GObject      *object,
 
   switch (property_id)
     {
-    case PROP_CLIENT:
-      grid->client = g_value_dup_object (value);
+    case PROP_APPLICATION:
+      grid->application = g_value_dup_object (value);
+      grid->client = gdu_application_get_client (grid->application);
       break;
 
     case PROP_BLOCK_OBJECT:
       gdu_volume_grid_set_block_object (grid, g_value_get_object (value));
       break;
 
+    case PROP_NO_MEDIA_STRING:
+      gdu_volume_grid_set_no_media_string (grid, g_value_get_string (value));
+      break;
+
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
       break;
@@ -496,11 +507,11 @@ gdu_volume_grid_class_init (GduVolumeGridClass *klass)
   gtkwidget_class->draw                 = gdu_volume_grid_draw;
 
   g_object_class_install_property (gobject_class,
-                                   PROP_CLIENT,
-                                   g_param_spec_object ("client",
-                                                        "Client",
-                                                        "The UDisksClient to use",
-                                                        UDISKS_TYPE_CLIENT,
+                                   PROP_APPLICATION,
+                                   g_param_spec_object ("application",
+                                                        "Application",
+                                                        "The GduApplication to use",
+                                                        GDU_TYPE_APPLICATION,
                                                         G_PARAM_READABLE |
                                                         G_PARAM_WRITABLE |
                                                         G_PARAM_CONSTRUCT_ONLY |
@@ -516,6 +527,17 @@ gdu_volume_grid_class_init (GduVolumeGridClass *klass)
                                                         G_PARAM_WRITABLE |
                                                         G_PARAM_STATIC_STRINGS));
 
+  g_object_class_install_property (gobject_class,
+                                   PROP_NO_MEDIA_STRING,
+                                   g_param_spec_string ("no-media-string",
+                                                        "No Media String",
+                                                        "The string to show when there is no media or block device",
+                                                        _("No Media"),
+                                                        G_PARAM_READABLE |
+                                                        G_PARAM_WRITABLE |
+                                                        G_PARAM_CONSTRUCT |
+                                                        G_PARAM_STATIC_STRINGS));
+
   signals[CHANGED_SIGNAL] = g_signal_new ("changed",
                                           GDU_TYPE_VOLUME_GRID,
                                           G_SIGNAL_RUN_LAST,
@@ -535,11 +557,11 @@ gdu_volume_grid_init (GduVolumeGrid *grid)
 }
 
 GtkWidget *
-gdu_volume_grid_new (UDisksClient *client)
+gdu_volume_grid_new (GduApplication *application)
 {
-  g_return_val_if_fail (UDISKS_IS_CLIENT (client), NULL);
+  g_return_val_if_fail (GDU_IS_APPLICATION (application), NULL);
   return GTK_WIDGET (g_object_new (GDU_TYPE_VOLUME_GRID,
-                                   "client", client,
+                                   "application", application,
                                    NULL));
 }
 
@@ -729,6 +751,7 @@ render_element (GduVolumeGrid *grid,
   GtkStateFlags state;
   GtkJunctionSides sides;
   GtkBorder border;
+  const gchar *markup;
 
   animate_spinner = FALSE;
 
@@ -856,7 +879,10 @@ render_element (GduVolumeGrid *grid,
 
   /* text */
   layout = pango_cairo_create_layout (cr);
-  pango_layout_set_markup (layout, element->markup != NULL ? element->markup : "", -1);
+  markup = element->markup;
+  if (markup == NULL)
+    markup = grid->no_media_string;
+  pango_layout_set_markup (layout, markup, -1);
   desc = pango_font_description_from_string ("Sans 7.0");
   pango_layout_set_font_description (layout, desc);
   pango_font_description_free (desc);
@@ -1283,21 +1309,9 @@ recompute_grid (GduVolumeGrid *grid)
   g_list_free (grid->elements);
   grid->elements = NULL;
 
-  //g_debug ("TODO: recompute grid for %s, container_visible=%d",
+  //g_debug ("TODO: recompute grid for %s",
   //         grid->block_object != NULL ?
-  //         g_dbus_object_get_object_path (grid->block_object) : "<nothing selected>",
-  //         grid->container_visible);
-
-  if (grid->container_visible)
-    {
-      element = g_new0 (GridElement, 1);
-      element->type = GDU_VOLUME_GRID_ELEMENT_TYPE_CONTAINER;
-      element->fixed_width = 40;
-      element->offset = 0;
-      element->size = 0;
-      element->markup = g_strdup (grid->container_markup);
-      grid->elements = g_list_append (grid->elements, element);
-    }
+  //         g_dbus_object_get_object_path (grid->block_object) : "<nothing selected>");
 
   if (grid->block_object == NULL)
     {
@@ -1322,9 +1336,9 @@ recompute_grid (GduVolumeGrid *grid)
   top_size = udisks_block_get_size (top_block);
 
   /* include "Free Space" elements if there is at least this much slack between
-   * partitions (currently 1% of the disk, at least 1MB)
+   * partitions (currently 1% of the disk, but at most 1MiB)
    */
-  free_space_slack = MAX (top_size / 100, 1000*1000);
+  free_space_slack = MIN (top_size / 100, 1024*1024);
 
   partitions = NULL;
   logical_partitions = NULL;
@@ -1549,7 +1563,7 @@ grid_element_set_details (GduVolumeGrid  *grid,
 
     case GDU_VOLUME_GRID_ELEMENT_TYPE_NO_MEDIA:
       {
-        element->markup = g_strdup (_("No Media"));
+        element->markup = NULL; /* means that grid->no_media_string will be used */
 
         if (grid->block_object != NULL)
           {
@@ -1602,6 +1616,7 @@ grid_element_set_details (GduVolumeGrid  *grid,
           element->show_configured = TRUE;
 
         jobs = udisks_client_get_jobs_for_object (grid->client, element->object);
+        jobs = g_list_concat (jobs, gdu_application_get_local_jobs_for_object (grid->application, element->object));
         element->show_spinner = (jobs != NULL);
         g_list_foreach (jobs, (GFunc) g_object_unref, NULL);
         g_list_free (jobs);
@@ -1667,7 +1682,32 @@ grid_element_set_details (GduVolumeGrid  *grid,
         else if (g_strcmp0 (usage, "raid") == 0 && strlen (label) > 0)
           {
             type_for_display = udisks_client_get_id_for_display (grid->client, usage, type, version, FALSE);
-            g_ptr_array_add (lines, g_strdup (label));
+            if (g_strcmp0 (type, "linux_raid_member") == 0)
+              {
+                const gchar *sep = strstr (label, ":");
+                if (sep != NULL && strlen (sep) > 1)
+                  {
+                    gchar *homehost = g_strndup (label, sep - label);
+                    g_ptr_array_add (lines,
+                                     /* Translators: Shown in volume grid for Linux RAID members. Please
+                                      *              keep this as short as possible.
+                                      *              The first %s is the array name (e.g. 'MirrorOnTheWall').
+                                      *              The second %s is the homehost (e.g. 'thinkpad').
+                                      */
+                                     g_strdup_printf (C_("volume-grid", "%s [local to %s]"),
+                                                      sep + 1,
+                                                      homehost));
+                    g_free (homehost);
+                  }
+                else
+                  {
+                    g_ptr_array_add (lines, g_strdup (label));
+                  }
+              }
+            else
+              {
+                g_ptr_array_add (lines, g_strdup (label));
+              }
             maybe_add_partition (grid, lines, partition);
             g_ptr_array_add (lines, g_strdup_printf ("%s %s", size_str, type_for_display));
             g_free (type_for_display);
@@ -1829,26 +1869,28 @@ on_client_changed (UDisksClient   *client,
 /* ---------------------------------------------------------------------------------------------------- */
 
 void
-gdu_volume_grid_set_container_visible (GduVolumeGrid  *grid,
-                                       gboolean        visible)
+gdu_volume_grid_set_no_media_string   (GduVolumeGrid       *grid,
+                                       const gchar         *str)
 {
   g_return_if_fail (GDU_IS_VOLUME_GRID (grid));
-  if (!!grid->container_visible != !!visible)
-    {
-      grid->container_visible = visible;
-      recompute_grid (grid);
-    }
+  if (g_strcmp0 (grid->no_media_string, str) == 0)
+    goto out;
+
+  g_free (grid->no_media_string);
+  grid->no_media_string = g_strdup (str);
+
+  g_object_notify (G_OBJECT (grid), "no-media-string");
+
+  gtk_widget_queue_draw (GTK_WIDGET (grid));
+
+ out:
+  ;
 }
 
-void
-gdu_volume_grid_set_container_markup (GduVolumeGrid  *grid,
-                                      const gchar    *markup)
+const gchar *
+gdu_volume_grid_get_no_media_string   (GduVolumeGrid      *grid)
 {
-  g_return_if_fail (GDU_IS_VOLUME_GRID (grid));
-  if (g_strcmp0 (grid->container_markup, markup) != 0)
-    {
-      g_free (grid->container_markup);
-      grid->container_markup = g_strdup (markup);
-      recompute_grid (grid);
-    }
+  g_return_val_if_fail (GDU_IS_VOLUME_GRID (grid), NULL);
+  return grid->no_media_string;
 }
+
