@@ -50,13 +50,30 @@ void            gdu_utils_show_error      (GtkWindow      *parent_window,
                                            const gchar    *message,
                                            GError         *error);
 
-gboolean        gdu_utils_show_confirmation (GtkWindow   *parent_window,
-                                             const gchar *message,
-                                             const gchar *secondary_message,
-                                             const gchar *affirmative_verb);
+gboolean        gdu_utils_show_confirmation (GtkWindow    *parent_window,
+                                             const gchar  *message,
+                                             const gchar  *secondary_message,
+                                             const gchar  *affirmative_verb,
+                                             const gchar  *checkbox_mnemonic,
+                                             gboolean     *inout_checkbox_value,
+                                             UDisksClient *client,
+                                             GList        *objects);
 
 gboolean gdu_utils_is_ntfs_available (void);
 
+gchar *gdu_utils_format_mdraid_level (const gchar *level,
+                                      gboolean     long_desc,
+                                      gboolean     use_markup);
+
+gboolean gdu_util_is_same_size (GList   *blocks,
+                                guint64 *out_min_size);
+
+gchar *gdu_utils_get_pretty_uri (GFile *file);
+
+gboolean gdu_utils_is_in_use (UDisksClient *client,
+                              UDisksObject *object);
+
+
 G_END_DECLS
 
 #endif /* __GDU_UTILS_H__ */
